<?php

/**
 * Payment method callback: Braintree Transparent Redirect settings form.
 */
function braintree_donations_api_settings_form($form, &$form_state) {
  $form = array();

  $settings = braintree_donations_api_settings();
  $form['settings'] = array(
    '#tree' => TRUE,
  );

  $form['settings']['merchant_id'] = array(
    '#type' => 'textfield',
    '#title' => t('Merchant ID'),
    '#default_value' => $settings['merchant_id'],
    '#required' => TRUE,
  );
  $form['settings']['public_key'] = array(
    '#type' => 'textfield',
    '#title' => t('Public key'),
    '#default_value' => $settings['public_key'],
    '#required' => TRUE,
  );
  $form['settings']['private_key'] = array(
    '#type' => 'textfield',
    '#title' => t('Private key'),
    '#default_value' => $settings['private_key'],
    '#required' => TRUE,
  );
  $form['settings']['environment'] = array(
    '#type' => 'radios',
    '#title' => t('Braintree server'),
    '#options' => array(
      'sandbox' => t('Sandbox - use for testing, requires a Braintree Sandbox account'),
      'production' => t('Production - use for processing real transactions'),
    ),
    '#default_value' => $settings['environment'],
  );
  $form['settings']['advanced_fraud_detection'] = array(
    '#type' => 'radios',
    '#title' => t('Send device data for advanced fraud detection'),
    '#options' => array(
      -1 => t('Force disable (regardless of Braintree control panel)'),
      0 => t('Disabled'),
      1 => t('Enabled'),
      2 => t('Automatically enabled'),
    ),
    '#description' => t('Enable this option before you enable advanced fraud detection in your Braintree account under the Processing Settings. It will be automatically enabled if you enable advanced fraud detection first in Braintree, but you risk a valid transaction being rejected.'),
    '#default_value' => $settings['advanced_fraud_detection'],
  );
  $form['buttons']['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Save configuration'),
  );
  return $form;
}

/**
 * Form validate function to verify the Braintree PHP library is available.
 *
 * @see braintree_donations_api_settings_form()
 */
function braintree_donations_api_settings_form_validate($form, &$form_state) {
  if (!file_exists(dirname(__FILE__) . '/vendor/autoload.php')) {
    form_set_error('settings', t('You must run composer - missing file vendor/autoload.php'));
  }
}

/**
 * Form submit function.
 *
 * @see braintree_donations_api_settings_form()
 */
function braintree_donations_api_settings_form_submit($form, &$form_state) {
  variable_set('braintree_donations_api_settings', $form_state['values']['settings']);
  drupal_set_message(t('The configuration options have been saved.'));
}

/**
 * Form builder function for customer settings.
 */
function braintree_donations_customer_settings_form($form, &$form_state) {
  if (braintree_donations_missing_settings($form)) {
    return $form;
  }
  $defaults = braintree_donations_customer_defaults();
  $form['defaults'] = array(
    '#tree' => TRUE,
  );
  $form['defaults']['locality'] = array(
    '#type' => 'textfield',
    '#title' => t('Default city'),
    '#default_value' => $defaults['locality'],
  );
  $form['defaults']['region'] = array(
    '#type' => 'textfield',
    '#title' => t('Default state'),
    '#default_value' => $defaults['region'],
    '#maxlength' => 4,
    '#size' => 4,
  );
  $form['defaults']['postal_code'] = array(
    '#type' => 'textfield',
    '#title' => t('Default ZIP code'),
    '#size' => 5,
    '#default_value' => $defaults['postal_code'],
  );
  $form['defaults']['country_code_alpha2'] = array(
    '#type' => 'textfield',
    '#title' => t('Default 2 Letter country code for all transactions'),
    '#description' => 'See https://developers.braintreepayments.com/javascript+php/reference/general/countries',
    '#default_value' => $defaults['country_code_alpha2'],
    '#required' => TRUE,
    '#maxlength' => 2,
    '#size' => 2,
  );
  $form['defaults']['country_code_user_choose'] = array(
    '#type' => 'checkbox',
    '#title' => t('Allow the user to choose a country (otherwise the default is always used)'),
    '#default_value' => $defaults['country_code_user_choose'],
  );
  $form['defaults']['country_code_user_choose'] = array(
    '#type' => 'checkbox',
    '#title' => t('Allow the user to choose a country (otherwise the default is always used)'),
    '#default_value' => $defaults['country_code_user_choose'],
  );
  $form['defaults']['country_codes_alpha2_priorities'] = array(
    '#type' => 'textfield',
    '#description' => t('Comma-separated list of 2 letter county codes that should be moved to the front of the list.'),
    '#title' => t('List of priority county codes'),
    '#default_value' => implode(', ', $defaults['country_codes_alpha2_priorities']),
  );
  $form['defaults']['country_codes'] = array(
    '#type' => 'fieldset',
    '#title' => t('List of county codes'),
    '#collapsed' => TRUE,
    '#collapsible' => TRUE,
  );
  $rows = array();
  foreach (braintree_donations_countries() as $code => $name) {
    $rows[] = array($code, $name);
  }
  $form['defaults']['country_codes']['table'] = array(
    '#theme' => 'table',
    '#header' => array(t('2 Letter Code'), t('County Name')),
    '#rows' => $rows,
  );
  $form['pages']['braintree_donations_require_billing_address'] = array(
    '#type' => 'checkbox',
    '#title' => t('Require billing address in addition to postal code'),
    '#default_value' => variable_get('braintree_donations_require_billing_address', TRUE),
  );
  $form['pages']['braintree_donations_enable_dropin_form'] = array(
    '#type' => 'checkbox',
    '#title' => t('Enable dropin form at <a href="@url">/braintree-dropin-donations</a>', array('@url' => url('braintree-dropin-donations'))),
    '#default_value' => variable_get('braintree_donations_enable_dropin_form', TRUE),
  );
  $form['pages']['braintree_donations_js_redirect'] = array(
    '#type' => 'textfield',
    '#title' => t('Donation Form Redirect'),
    '#description' => t('Path where form submissions should be redirected on success.'),
    '#default_value' => variable_get('braintree_donations_js_redirect', '<front>'),
  );
  $form['buttons']['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Save defaults'),
  );
  return $form;
}

/**
 * Form validate function for customer settings.
 *
 * @see braintree_donations_customer_settings_form()
 */
function braintree_donations_customer_settings_form_validate($form, &$form_state) {
  $countries = braintree_donations_countries();
  $code = $form_state['values']['defaults']['country_code_alpha2'];
  if (empty($countries[$code])) {
    form_set_error("defaults][country_code_alpha2", t('Invalid county code %code.', array('%code' => $code)));
  }
  $priorities = explode(',', $form_state['values']['defaults']['country_codes_alpha2_priorities']);
  $form_state['values']['defaults']['country_codes_alpha2_priorities'] = array();
  foreach ($priorities as $code) {
    $code = trim($code);
    if (empty($countries[$code])) {
      form_set_error("defaults][country_codes_alpha2_priorities", t('Invalid county code %code.', array('%code' => $code)));
    }
    else {
      // Transform this into an array of codes.
      $form_state['values']['defaults']['country_codes_alpha2_priorities'][$code] = $code;
    }
  }
}

/**
 * Form submit function for customer settings.
 *
 * @see braintree_donations_customer_settings_form()
 */
function braintree_donations_customer_settings_form_submit($form, &$form_state) {
  variable_set('braintree_donations_customer_defaults', $form_state['values']['defaults']);
  variable_set('braintree_donations_require_billing_address', $form_state['values']['braintree_donations_require_billing_address']);
  variable_set('braintree_donations_enable_dropin_form', $form_state['values']['braintree_donations_enable_dropin_form']);
  variable_set('braintree_donations_js_redirect', $form_state['values']['braintree_donations_js_redirect']);
  drupal_set_message(t('The configuration options have been saved.'));
}

/**
 * Page callback for the recurring donations admin page.
 *
 * @return array
 *   A render array with forms.
 */
function braintree_donations_recurring_admin_page() {
  $build = array();
  $failed_queue = DrupalQueue::get('braintree_donations_recurring_failed', TRUE);
  $failed_queue->createQueue();
  $count = $failed_queue->numberOfItems();
  if ($count > 0) {
    $build[] = drupal_get_form('braintree_donations_recurring_requeue_form');
    // @todo use format_plural() for better grammar.
    drupal_set_message(t('There are @count transactions in the %fail_queue queue that failed in creating a recurring charge.', array('@count' => $count, '%fail_queue' => 'braintree_donations_recurring_failed')), 'error');
  }
  $build[] = drupal_get_form('braintree_donations_recurring_settings_form');
  return $build;
}

/**
 * Form builder function for retrying failed transactions.
 */
function braintree_donations_recurring_requeue_form($form, &$form_state) {
  $form['buttons']['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Re-try failed recurring transactions'),
  );
  return $form;
}

/**
 * Form submit function for retrying failed transactions.
 *
 * @see braintree_donations_recurring_requeue_form()
 */
function braintree_donations_recurring_requeue_form_submit($form, &$form_state) {
  /* @var \DrupalQueueInterface $failed_queue  */
  $failed_queue = DrupalQueue::get('braintree_donations_recurring_failed', TRUE);
  /* @var \DrupalQueueInterface $queue  */
  $queue = DrupalQueue::get('braintree_donations_recurring', TRUE);
  $queue->createQueue();
  $count = 0;
  while ($item = $failed_queue->claimItem()) {
    $transaction = $item->data;
    $transaction['retry_at'] = 0;
    $queue->createItem($transaction);
    $failed_queue->deleteItem($item);
    $count++;
  }
  // @todo use format_plural().
  drupal_set_message(t('@count previously failed items were moved back from the %fail_queue queue.', array('@count' => $count, '%fail_queue' => 'braintree_donations_recurring_failed')));
}

/**
 * Form builder function for recurring payment settings.
 */
function braintree_donations_recurring_settings_form($form, &$form_state) {
  if (braintree_donations_missing_settings($form)) {
    return $form;
  }
  $plans = array();
  try {
    // Re-key plan data by the ID.
    foreach (Braintree_Plan::all() as $plan) {
      $plans[$plan->id] = $plan;
    }
  }
  catch (Braintree_Exception $e) {
    braintree_donations_handle_exception($e);
    return $form;
  }

  $defaults = braintree_donations_recurring_settings();

  $form['#plan_data'] = $plans;
  $form['plans'] = array(
    '#tree' => TRUE,
    '#type' => 'fieldset',
    '#title' => t('Available recurring donation plans')
  );
  foreach ($plans as $plan) {
    $form['plans'][$plan->id] = array(
      '#title' => check_plain($plan->name),
      '#type' => 'checkbox',
      '#description' => t('%description plan with a billing cycle of every @count months', array('%description' => $plan->description, '@count' => $plan->billingFrequency)),
      '#default_value' => !empty($defaults['plans'][$plan->id]),
    );
  }
  $form['braintree_donations_queue_on_cron'] = array(
    '#type' => 'checkbox',
    '#title' => t('Enable processing of recurring on Drupal cron. Un-check if you are processing using the drush command.'),
    '#default_value' => variable_get('braintree_donations_queue_on_cron', TRUE),
  );

  $form['buttons']['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Save settings'),
  );
  return $form;
}

/**
 * Form submit function for recurring payment settings.
 */
function braintree_donations_recurring_settings_form_submit($form, &$form_state) {
  $plans = array_filter($form_state['values']['plans']);
  $frequencies = array();
  // Save the plan names for use in donation forms.
  foreach($plans as $id => $v) {
    $plans[$id] = check_plain($form['#plan_data'][$id]->name);
    $frequencies[$id] = $form['#plan_data'][$id]->billingFrequency;
  }

  variable_set('braintree_donations_recurring_settings', array('plans' => $plans, 'frequencies' => $frequencies));
  variable_set('braintree_donations_queue_on_cron', (bool) $form_state['values']['braintree_donations_queue_on_cron']);
  drupal_set_message(t('The configuration options have been saved.'));
}
